/*
** fdgetdelim.c - part of GrabURL 2.0
** Copyright (C) 1996-97 Serge Emond
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

/*
** Simple function to read characters from a file descriptor,
** stripping the chars in "strip" and stopping to read after the
** first char in "end".
**
** Adds '\0' to the end of the buf...
**
** Return:
**  -1 on error
**  0 on eof/empty line/etc..
**  else number of bytes in buffer
*/

int fdgetdelim(int fd, char *buf, int bsize, const char *strip, const char *end)
{
	int got, igot, i, addit;
	char tmpc;

	got = 0;

	while (1) {
	  if (got >= bsize-1) break; /* buffer full */
	  igot = read(fd, &tmpc, 1);
	  if (igot == 0) break;  /* eof */
	  if (igot == -1) return (-1); /* error */

	  addit = 1;

	  if (end)
		for (i=0; end[i]; i++)
		  if (end[i] ==   tmpc) {
			addit = -1;
			break;
		  }

	  if (strip)
		for (i=0; strip[i]; i++)
		  if (strip[i] == tmpc) {
			addit = 0;
			break;
		  }

	  if (addit)
		buf[got++] = tmpc;

	  if (addit<0)
		break;

	}

	buf[got]=0;
	return(got);
}
